package cz.muni.cv4storage;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

/**
 * https://github.com/codepath/android_guides/wiki/Local-Databases-with-SQLiteOpenHelper
 */
public class PostsDatabaseHelper extends SQLiteOpenHelper {

	/********************
	 * Database creation
	 ********************/
	// Database Info
	private static final String DATABASE_NAME = "postsDatabase";
	private static final int DATABASE_VERSION = 1;

	// Table Names
	private static final String TABLE_POSTS = "posts";
	private static final String TABLE_USERS = "users";

	// Post Table Columns
	private static final String KEY_POST_ID = "id";
	private static final String KEY_POST_USER_ID_FK = "userId";
	private static final String KEY_POST_TEXT = "text";

	// User Table Columns
	private static final String KEY_USER_ID = "id";
	private static final String KEY_USER_NAME = "userName";
	private static final String KEY_USER_PROFILE_PICTURE_URL = "profilePictureUrl";

	public PostsDatabaseHelper(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	// Called when the database connection is being configured.
	// Configure database settings for things like foreign key support, write-ahead logging, etc.
	@Override
	public void onConfigure(SQLiteDatabase db) {
		super.onConfigure(db);
		db.setForeignKeyConstraintsEnabled(true);
	}

	// Called when the database is created for the FIRST time.
	// If a database already exists on disk with the same DATABASE_NAME, this method will NOT be called.
	@Override
	public void onCreate(SQLiteDatabase db) {
		String CREATE_POSTS_TABLE = "CREATE TABLE " + TABLE_POSTS +
				"(" +
				KEY_POST_ID + " INTEGER PRIMARY KEY," + // Define a primary key
				KEY_POST_USER_ID_FK + " INTEGER REFERENCES " + TABLE_USERS + "," + // Define a foreign key
				KEY_POST_TEXT + " TEXT" +
				")";

		String CREATE_USERS_TABLE = "CREATE TABLE " + TABLE_USERS +
				"(" +
				KEY_USER_ID + " INTEGER PRIMARY KEY," +
				KEY_USER_NAME + " TEXT," +
				KEY_USER_PROFILE_PICTURE_URL + " TEXT" +
				")";

		db.execSQL(CREATE_POSTS_TABLE);
		db.execSQL(CREATE_USERS_TABLE);
	}

	// Called when the database needs to be upgraded.
	// This method will only be called if a database already exists on disk with the same DATABASE_NAME,
	// but the DATABASE_VERSION is different than the version of the database that exists on disk.
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		if (oldVersion != newVersion) {
			// Simplest implementation is to drop all old tables and recreate them
			db.execSQL("DROP TABLE IF EXISTS " + TABLE_POSTS);
			db.execSQL("DROP TABLE IF EXISTS " + TABLE_USERS);
			onCreate(db);
		}
	}

	/***************
	 * Access to DB
	 ***************/

	private static PostsDatabaseHelper sInstance;

	public static synchronized PostsDatabaseHelper getInstance(Context context) {
		// Use the application context, which will ensure that you
		// don't accidentally leak an Activity's context.
		// See this article for more information: http://bit.ly/6LRzfx
		if (sInstance == null) {
			sInstance = new PostsDatabaseHelper(context.getApplicationContext());
		}
		return sInstance;
	}

	public class Post {
		public User user;
		public String text;
	}

	public class User {
		public String userName;
		public String profilePictureUrl;
	}

	// Insert a post into the database
	public void addPost(Post post) {
		// Create and/or open the database for writing
		SQLiteDatabase db = getWritableDatabase();

		// It's a good idea to wrap our insert in a transaction. This helps with performance and ensures
		// consistency of the database.
		db.beginTransaction();
		try {
			// The user might already exist in the database (i.e. the same user created multiple posts).
			long userId = addOrUpdateUser(post.user);

			ContentValues values = new ContentValues();
			values.put(KEY_POST_USER_ID_FK, userId);
			values.put(KEY_POST_TEXT, post.text);

			// Notice how we haven't specified the primary key. SQLite auto increments the primary key column.
			db.insertOrThrow(TABLE_POSTS, null, values);
			db.setTransactionSuccessful();
		} catch (Exception e) {
			// TODO Error while trying to add post to database
		} finally {
			db.endTransaction();
		}
	}

	// Insert or update a user in the database
	// Since SQLite doesn't support "upsert" we need to fall back on an attempt to UPDATE (in case the
	// user already exists) optionally followed by an INSERT (in case the user does not already exist).
	// Unfortunately, there is a bug with the insertOnConflict method
	// (https://code.google.com/p/android/issues/detail?id=13045) so we need to fall back to the more
	// verbose option of querying for the user's primary key if we did an update.
	public long addOrUpdateUser(User user) {
		// The database connection is cached so it's not expensive to call getWriteableDatabase() multiple times.
		SQLiteDatabase db = getWritableDatabase();
		long userId = -1;

		db.beginTransaction();
		try {
			ContentValues values = new ContentValues();
			values.put(KEY_USER_NAME, user.userName);
			values.put(KEY_USER_PROFILE_PICTURE_URL, user.profilePictureUrl);

			// First try to update the user in case the user already exists in the database
			// This assumes userNames are unique
			int rows = db.update(TABLE_USERS, values, KEY_USER_NAME + "= ?", new String[]{user.userName});

			// Check if update succeeded
			if (rows == 1) {
				// Get the primary key of the user we just updated
				String usersSelectQuery = String.format("SELECT %s FROM %s WHERE %s = ?",
						KEY_USER_ID, TABLE_USERS, KEY_USER_NAME);
				Cursor cursor = db.rawQuery(usersSelectQuery, new String[]{String.valueOf(user.userName)});
				try {
					if (cursor.moveToFirst()) {
						userId = cursor.getInt(0);
						// cursor.getString(cursor.getColumnIndex("Name"))
						db.setTransactionSuccessful();
					}
				} finally {
					if (cursor != null && !cursor.isClosed()) {
						cursor.close();
					}
				}
			} else {
				// user with this userName did not already exist, so insert new user
				userId = db.insertOrThrow(TABLE_USERS, null, values);
				db.setTransactionSuccessful();
			}
		} catch (Exception e) {
			// TODO Error while trying to add or update user
		} finally {
			db.endTransaction();
		}
		return userId;
	}
}
